""" webfilter/Blocker.py

    Copyright (C) 2000,2001  Bastian Kleineidam

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
import re,string,urlparse,webfilter
from webfilter import debug
from get_dom import *
from webfilter.Rules import Netlocparts
from WebCleanerConf import config_dir
orders = [webfilter.FILTER_REQUEST]
# regular expression for image filenames
image_re=re.compile(r'\.(?i)(gif|jpg|png)')

def strblock(block):
    s="("
    for b in block:
        s = s + ","+(b and b.pattern or "")
    return s+")"

class Blocker(webfilter.Filter):
    def __init__(self):
        """With no blocker and no allower we never block.
        """
        from os.path import join
        self.block = []
        self.allow = []
        self.blocked_url="file://"+join(config_dir, "blocked.html")
        self.blocked_image="file://"+join(config_dir, "blocked.gif")


    def configure(self, dom):
        self._add_to_list(dom, 'block')
        self._add_to_list(dom, 'allow')


    def _add_to_list(self, dom, name):
        rulelist = getattr(self, name)
        for node in dom.getElementsByTagName(name):
            attrs = get_node_attrs(node)
            title = get_attr(attrs, 'title')
            if int(get_node_attr(node, 'disable')):
                debug("disable %s '%s'" % (name,title), 2)
	        continue
            debug("enable %s '%s'" % (name,title), 2)
            rule = []
            for part in Netlocparts:
                rule.append(get_node_attr(node, part))
            rule = map(lambda x: x and re.compile(x) or None, rule)
            # append the block url (if any)
            rule.append(node_value(node) or "")
            rulelist.append(rule)


    def filter(self, data, **args):
        splitted = string.split(data)
        if len(splitted)==3:
            method,url,protocol = splitted
            urlTuple = list(urlparse.urlparse(url))
            netloc = urlTuple[1]
            s = string.split(netloc, ":")
            if len(s)==2:
                urlTuple[1:2] = s
            else:
                urlTuple[1:2] = [netloc,80]
            blocked = self.blocked(urlTuple)
            if blocked is not None:
                debug("blocked url %s" % url, 2)
                if image_re.match(url[-4:]):
                    return '%s %s %s' % (method,
		           blocked or self.blocked_image, 'image/gif')
                else:
                    return '%s %s %s' % (method,
                           blocked or self.blocked_url, 'text/html')
        return data


    def blocked(self, urlTuple):
        debug("checking block", 3)
        for _block in self.block:
            match = 1
            for i in range(len(urlTuple)):
                if _block[i]:
                    debug("block pattern "+_block[i].pattern, 3)
                    if not _block[i].search(urlTuple[i]):
                        debug("no match", 3)
                        match = 0
            if match and not self.allowed(urlTuple):
                return _block[-1]
        return None


    def allowed(self, urlTuple):
        debug("checking allow", 3)
        for _allow in self.allow:
            match = 1
            for i in range(len(urlTuple)):
                if _allow[i]:
                    debug("allow pattern "+_allow[i].pattern, 3)
		    if not _allow[i].search(urlTuple[i]):
                        debug("no match", 3)
                        match = 0
            if match: return 1
        return 0
