""" webfilter/daemon.py

    Copyright (C) 2000,2001  Bastian Kleineidam

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

We define helper functions to start, stop, restart and reload daemons.
Of course this is OS dependent and currently we support only Unix.
"""
import os

if os.name=='posix':
    kill = os.kill
elif os.name=='nt':
    def kill(pid, sig):
        """kill function for Win32"""
        import win32api
        handle = win32api.OpenProcess(1, 0, pid)
        return (0 != win32api.TerminateProcess(handle, 0))
        

def iswriteable(file):
    if os.path.isdir(file): return 0
    if os.path.islink(file): return 0
    try:
        if os.path.exists(file):
            f = open(file, 'a')
            f.close()
            return 1
        else:
            f = open(file, 'w')
            f.close()
            os.remove(file)
            return 1
    except IOError:
        pass
    return 0


pidfile='/var/run/webcleaner.pid'
if not iswriteable(pidfile):
    pidfile = '/var/tmp/webcleaner.pid'
if not iswriteable(pidfile):
    pidfile = '/tmp/webcleaner.pid'
if not iswriteable(pidfile):
    pidfile = 'webcleaner.pid'


def start(startfunc):
    """start a daemon using the appropriate pidfile"""
    if os.name!='posix':
        print "Sorry, non-POSIX systems not yet supported"
        return
    # already running?
    if os.path.exists(pidfile): return
    # forking (only under POSIX systems)
    pid = os.fork()
    # the parent exits
    if pid!=0:
        raise SystemExit()
    # write pid in pidfile
    f = open(pidfile, 'w')
    f.write("%d" % os.getpid())
    f.close()
    # starting
    startfunc()


def stop():
    if os.name!='posix':
        print "Sorry, non-POSIX systems not yet supported"
        return
    if not os.path.exists(pidfile): return
    import signal
    os.kill(int(open(pidfile).read()), signal.SIGTERM)
    os.remove(pidfile)


def restart(startfunc):
    if os.name!='posix':
        print "Sorry, non-POSIX systems not yet supported"
        return
    stop()
    # sleep 3 seconds, should be enough to clean up
    import time
    time.sleep(3)
    start(startfunc)


def reload():
    if os.name!='posix':
        print "Sorry, non-POSIX systems not yet supported"
        return
    if not os.path.exists(pidfile): return
    pid = int(open(pidfile).read())
    import signal
    os.kill(int(open(pidfile).read()), signal.SIGHUP)


def status():
    if os.name!='posix':
        print "Sorry, non-POSIX systems not yet supported"
        return
    if os.path.exists(pidfile):
        pid = open(pidfile).read()
        return "webcleaner is running (PID %s)" % pid
    else:
        return "webcleaner is not running"
